SystemOrganization addCategory: #'Athens-Text'!

FT2GlyphSlot subclass: #SimpleTextGlyphInfo
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Text'!

!SimpleTextGlyphInfo classSide methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 7/13/2011 04:04'!
fromFace: anFT2Face
	^(self new)
		loadFrom: anFT2Face;
		yourself.! !

!SimpleTextGlyphInfo methodsFor: 'accessing' stamp: 'IgorStasenko 7/13/2011 04:05'!
face
	^ face! !

!SimpleTextGlyphInfo methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 10/20/2011 05:09'!
loadFrom: anFT2Face
	"Same as in superclass, but avoid converting coordinates to not waste CPU cycles"
	
	face := anFT2Face.
	self primLoadFrom: anFT2Face.
	format := ((Smalltalk isLittleEndian) ifTrue: [ format reversed ] ifFalse: [ format ]) asString.
! !

!FreeTypeFont methodsFor: '*Athens-Text' stamp: 'IgorStasenko 11/7/2011 14:15'!
getGlyphWidth: aCharacter 
	aCharacter asciiValue = 9 ifTrue: [ 
		^  (self linearWidthOf: Character space) * 8
		]. 
	^ (self linearWidthOf: aCharacter)! !

!FreeTypeFont methodsFor: '*Athens-Text' stamp: 'IgorStasenko 10/28/2011 12:28'!
getPreciseAscent
	"we using face bouding box for getting ascent value, otherwise two different
	font sizes on same line won't align properly"
	
	^(self face bbox bottom * self pixelSize / self face unitsPerEm) asFloat.
! !

!FreeTypeFont methodsFor: '*Athens-Text' stamp: 'IgorStasenko 11/7/2011 14:47'!
getPreciseDescent
	"we using face bouding box for getting ascent value, otherwise two different
	font sizes on same line won't align properly"
	
	^(self face descender * self pixelSize / self face unitsPerEm) asFloat negated.

! !

!FreeTypeFont methodsFor: '*Athens-Text' stamp: 'IgorStasenko 10/28/2011 11:46'!
getPreciseHeight
	"get precise font ascent, can be floating value"
	
	^(self face height * self pixelSize / self face unitsPerEm) asFloat.
! !

!FreeTypeFont methodsFor: '*Athens-Text' stamp: 'IgorStasenko 10/20/2011 05:12'!
glyphRendererOn: anAthensSurface

	^ anAthensSurface getFreetypeFontRendererFor: self
! !

TextAttribute subclass: #AthensTextBackground
	instanceVariableNames: 'color'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Text'!

!AthensTextBackground methodsFor: 'accessing' stamp: 'IgorStasenko 10/28/2011 14:42'!
color

	^ color! !

!AthensTextBackground methodsFor: 'accessing' stamp: 'IgorStasenko 10/28/2011 14:42'!
color: anObject

	color := anObject! !

!AthensTextBackground methodsFor: 'scanning' stamp: 'IgorStasenko 10/28/2011 14:31'!
dominates: other
	^ other class == self class! !

!AthensTextBackground methodsFor: 'scanning' stamp: 'IgorStasenko 10/28/2011 14:31'!
emphasizeScanner: scanner
	"Set the emphasis for text display"
	scanner textBackground: color! !

!AthensTextBackground methodsFor: 'comparing' stamp: 'IgorStasenko 10/28/2011 14:31'!
hash
	^ color hash! !

!AthensTextBackground methodsFor: 'printing' stamp: 'IgorStasenko 10/28/2011 14:32'!
printOn: aStream
	super printOn: aStream.
	aStream nextPutAll: ' code: '; print: color! !

Object subclass: #AthensTextComposer
	instanceVariableNames: 'lines text'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Text'!

!AthensTextComposer classSide methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 10/11/2011 16:07'!
scan: aText for: scanner
	^ self new scan: aText for: scanner! !

!AthensTextComposer methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 11/21/2011 12:51'!
lines
	^ lines! !

!AthensTextComposer methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 10/17/2011 12:42'!
renderOn: aCanvas

	| renderer |
	
	renderer := AthensTextRenderer new.
	renderer render: text lines: lines on: aCanvas.
! !

!AthensTextComposer methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 10/30/2011 19:14'!
scan: aText for: scanner
	" scan text from starting index,
	answer the index of first character for potential next line"
	| index top |	

	text := aText.
	scanner text: text.

	"edge case, text is empty "
	text isEmpty ifTrue: [ ^ scanner ].
	
	scanner newLine: 1.
	index := 1.

	text runs withStartStopAndValueDo: [:start :stop :values |
		
		scanner setAttributes: values.
		start to: stop do: [:i |  | ch |
			ch := text at: i.
			ch = Character cr 
				ifTrue: [ 
					scanner endLine: i - 1.
					scanner newLine: i + 1 ]
				ifFalse: [
					scanner addCharacter: i.
				]
			]
	].
	scanner endLine: (text size).

	lines := scanner lines.

	"set the lines top offset"
	top := 0.
	lines do: [:line |
		line top: top.
		top := top + line height ].
	
! !

Object subclass: #AthensTextDisplayCommand
	instanceVariableNames: 'next'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Text'!

!AthensTextDisplayCommand commentStamp: 'IgorStasenko 2/6/2012 16:20' prior: 0!
An abstract base class for all text rendering commands.

A text rendered takes a stream (or list) of my (sub)instances and in that way it renders the text!

AthensTextDisplayCommand subclass: #AthensBackgroundChange
	instanceVariableNames: 'color start width'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Text'!

!AthensBackgroundChange commentStamp: 'IgorStasenko 2/6/2012 16:16' prior: 0!
Text display command for changing the background color!

!AthensBackgroundChange methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 10/28/2011 14:44'!
addWidth: aWidth
	width := width + aWidth! !

!AthensBackgroundChange methodsFor: 'accessing' stamp: 'IgorStasenko 10/28/2011 14:33'!
color

	^ color! !

!AthensBackgroundChange methodsFor: 'accessing' stamp: 'IgorStasenko 10/28/2011 14:33'!
color: anObject

	color := anObject! !

!AthensBackgroundChange methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 10/28/2011 14:45'!
initialize
	width :=0.
	color := Color transparent.! !

!AthensBackgroundChange methodsFor: 'rendering' stamp: 'IgorStasenko 11/7/2011 13:42'!
renderOn: renderer
	color isTransparent ifFalse: [
		renderer renderBackground: self
	]! !

!AthensBackgroundChange methodsFor: 'accessing' stamp: 'IgorStasenko 11/7/2011 13:41'!
start
	^ start! !

!AthensBackgroundChange methodsFor: 'accessing' stamp: 'IgorStasenko 11/7/2011 13:41'!
start: aStart
	start := aStart! !

!AthensBackgroundChange methodsFor: 'accessing' stamp: 'IgorStasenko 10/28/2011 14:33'!
width

	^ width! !

!AthensBackgroundChange methodsFor: 'accessing' stamp: 'IgorStasenko 10/28/2011 14:33'!
width: anObject

	width := anObject! !

AthensTextDisplayCommand subclass: #AthensCharacterSpan
	instanceVariableNames: 'start stop'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Text'!

!AthensCharacterSpan commentStamp: 'IgorStasenko 2/6/2012 16:17' prior: 0!
A text command to render a character span containing the range of characters for with same attributes!

!AthensCharacterSpan methodsFor: 'rendering' stamp: 'IgorStasenko 10/17/2011 12:52'!
renderOn: renderer
	renderer renderCharactersFrom: start to: stop! !

!AthensCharacterSpan methodsFor: 'accessing' stamp: 'IgorStasenko 11/10/2011 10:31'!
splitAt: position
	| newSpan |
	"split receiver on two character spans "
	" position points to first character which should be included in newly created span "
	
	position == start ifTrue: [ ^ self ].
	(position < start or: [ position > stop ]) ifTrue: [ self error: 'invalid position' ].

	newSpan := self class new
		start: position;
		stop: stop.

	stop := position - 1.
	self insert: newSpan.! !

!AthensCharacterSpan methodsFor: 'accessing' stamp: 'IgorStasenko 10/17/2011 12:02'!
start

	^ start! !

!AthensCharacterSpan methodsFor: 'accessing' stamp: 'IgorStasenko 10/17/2011 12:02'!
start: anObject

	start := anObject! !

!AthensCharacterSpan methodsFor: 'accessing' stamp: 'IgorStasenko 10/17/2011 12:02'!
stop

	^ stop! !

!AthensCharacterSpan methodsFor: 'accessing' stamp: 'IgorStasenko 10/17/2011 12:02'!
stop: anObject

	stop := anObject! !

AthensTextDisplayCommand subclass: #AthensColorChange
	instanceVariableNames: 'color'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Text'!

!AthensColorChange commentStamp: 'IgorStasenko 2/6/2012 16:17' prior: 0!
A command to change the current font color!

!AthensColorChange methodsFor: 'accessing' stamp: 'IgorStasenko 10/11/2011 16:58'!
color

	^ color! !

!AthensColorChange methodsFor: 'accessing' stamp: 'IgorStasenko 10/11/2011 16:58'!
color: anObject

	color := anObject! !

!AthensColorChange methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 10/17/2011 12:51'!
renderOn: renderer
	renderer setCurrentColor: color! !

AthensTextDisplayCommand subclass: #AthensFontChange
	instanceVariableNames: 'font'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Text'!

!AthensFontChange commentStamp: 'IgorStasenko 2/6/2012 16:17' prior: 0!
A command to change the currently selected font!

!AthensFontChange classSide methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 10/11/2011 16:55'!
font: aFont 
	^ self new font: aFont! !

!AthensFontChange methodsFor: 'accessing' stamp: 'IgorStasenko 11/10/2011 11:07'!
font
	^ font! !

!AthensFontChange methodsFor: 'accessing' stamp: 'IgorStasenko 10/11/2011 16:55'!
font: aFont
	font := aFont! !

!AthensFontChange methodsFor: 'rendering' stamp: 'IgorStasenko 10/17/2011 12:50'!
renderOn: anAthensTextRenderer 
	anAthensTextRenderer setCurrentFont: font.! !

!AthensTextDisplayCommand methodsFor: 'accessing' stamp: 'IgorStasenko 11/10/2011 10:14'!
addLast: aCommand
	self tail next: aCommand! !

!AthensTextDisplayCommand methodsFor: 'accessing' stamp: 'IgorStasenko 11/10/2011 10:21'!
do: aBlock
	| nn |
	nn := self.
	
	[ nn isNil ] whileFalse: [ 
		aBlock value: nn.
		nn := nn next. ]! !

!AthensTextDisplayCommand methodsFor: 'accessing' stamp: 'IgorStasenko 11/10/2011 10:19'!
insert: aCommands

	| nn |
	nn := next.
	next := aCommands.
	next tail next: nn! !

!AthensTextDisplayCommand methodsFor: 'accessing' stamp: 'IgorStasenko 11/10/2011 10:12'!
next

	^ next! !

!AthensTextDisplayCommand methodsFor: 'accessing' stamp: 'IgorStasenko 11/10/2011 10:12'!
next: anObject
	
	next := anObject! !

!AthensTextDisplayCommand methodsFor: 'copying' stamp: 'IgorStasenko 11/10/2011 10:47'!
postCopy
	"unlink the copy by default"
	next := nil! !

!AthensTextDisplayCommand methodsFor: 'rendering' stamp: 'IgorStasenko 10/30/2011 18:57'!
renderOn: renderer
	self shouldBeImplemented ! !

!AthensTextDisplayCommand methodsFor: 'accessing' stamp: 'IgorStasenko 11/10/2011 10:15'!
tail
	| nn |
	nn := self.
	[ nn next notNil ] whileTrue: [ nn := nn next ].
	^ nn! !

Object subclass: #AthensTextLine
	instanceVariableNames: 'text startIndex endIndex height maxAscent maxDescent width top commands tail'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Text'!

!AthensTextLine commentStamp: 'IgorStasenko 2/6/2012 16:20' prior: 0!
I representing a single line of text!

!AthensTextLine methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 11/10/2011 10:20'!
addCommand: aCommand

	commands 
		ifNil: [ tail := commands := aCommand. ]
		ifNotNil: [ tail next: aCommand ].
	tail := tail tail.
! !

!AthensTextLine methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 10/28/2011 12:43'!
addWidth: aNumber
	width := width + aNumber.! !

!AthensTextLine methodsFor: 'accessing' stamp: 'IgorStasenko 11/21/2011 12:47'!
bottom
	^ top + self heightAndDescent! !

!AthensTextLine methodsFor: 'accessing' stamp: 'IgorStasenko 10/11/2011 17:19'!
commands

	^ commands! !

!AthensTextLine methodsFor: 'accessing' stamp: 'IgorStasenko 10/11/2011 17:19'!
commands: anObject

	commands := anObject! !

!AthensTextLine methodsFor: 'accessing' stamp: 'IgorStasenko 10/11/2011 17:19'!
endIndex

	^ endIndex! !

!AthensTextLine methodsFor: 'accessing' stamp: 'IgorStasenko 10/11/2011 17:19'!
endIndex: anObject

	endIndex := anObject! !

!AthensTextLine methodsFor: 'accessing' stamp: 'IgorStasenko 10/5/2011 18:04'!
height
	^ height! !

!AthensTextLine methodsFor: 'accessing' stamp: 'IgorStasenko 10/11/2011 17:19'!
height: anObject

	height := anObject! !

!AthensTextLine methodsFor: 'accessing' stamp: 'IgorStasenko 11/7/2011 14:44'!
heightAndDescent

	^ height + maxDescent! !

!AthensTextLine methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 11/10/2011 10:15'!
initialize
	commands := tail := nil.
	width := height := maxAscent := maxDescent := 0.! !

!AthensTextLine methodsFor: 'accessing' stamp: 'IgorStasenko 10/28/2011 11:00'!
maxAscent
	^ maxAscent! !

!AthensTextLine methodsFor: 'accessing' stamp: 'IgorStasenko 10/28/2011 11:05'!
maxAscent: mascent
	maxAscent := mascent max: maxAscent! !

!AthensTextLine methodsFor: 'accessing' stamp: 'IgorStasenko 11/7/2011 14:36'!
maxDescent
	^ maxDescent! !

!AthensTextLine methodsFor: 'accessing' stamp: 'IgorStasenko 11/7/2011 14:43'!
maxDescent: mascent

	maxDescent := mascent max: maxDescent! !

!AthensTextLine methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 10/28/2011 12:41'!
maxHeight: aHeight
	height := height max: aHeight! !

!AthensTextLine methodsFor: 'rendering' stamp: 'IgorStasenko 11/10/2011 12:23'!
renderOn: aTextRenderer
	
	commands ifNil: [ ^ self ]. 
	commands do: [ :cmd |
		cmd renderOn: aTextRenderer
	].
! !

!AthensTextLine methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 10/20/2011 10:30'!
renderOn: anAthensTextRenderer x: aSmallInteger y: aSmallInteger3 
	self shouldBeImplemented.! !

!AthensTextLine methodsFor: 'accessing' stamp: 'IgorStasenko 10/11/2011 17:19'!
startIndex

	^ startIndex! !

!AthensTextLine methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 10/11/2011 16:10'!
startIndex: index
	startIndex := index! !

!AthensTextLine methodsFor: 'accessing' stamp: 'IgorStasenko 10/11/2011 17:19'!
text

	^ text! !

!AthensTextLine methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 10/11/2011 16:48'!
text: aText 
	text := aText! !

!AthensTextLine methodsFor: 'accessing' stamp: 'IgorStasenko 11/21/2011 12:47'!
top
	^ top! !

!AthensTextLine methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 10/5/2011 18:05'!
top: aTop
	top := aTop! !

!AthensTextLine methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 10/5/2011 18:04'!
width
	^ width! !

!AthensTextLine methodsFor: 'accessing' stamp: 'IgorStasenko 10/11/2011 17:19'!
width: anObject

	width := anObject! !

Object subclass: #AthensTextRenderTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Text'!

!AthensTextRenderTest classSide methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 10/17/2011 14:34'!
test1
	|  t  c surf |

t := '
AthensTextRenderTest test1

leftFlush
p := MultiNewParagraph new.

p compose: t
	style: TextStyle default
	from: 1
	in: (0@0 corner: 100@100).
p

MultiNewParagraph
TextStyle default alignment


globalBounds:  toLocal: 
' 
asText .

	t addAttribute: (TextColor new color: Color red) from: 3 to: 10.
	t addAttribute: (TextEmphasis italic) from: 5 to: 13.

	c := AthensTextComposer scan: t for: AthensTextScanner new.

	surf := AthensBalloonSurface new form: (Form extent: Display extent depth: 32).

	surf drawDuring: [ :canvas | 
		canvas pathTransform scaleBy: 0.8.
		
		c renderOn: canvas
	].

	Display getCanvas translucentImage: surf form at: 0@0

! !

!AthensTextRenderTest classSide methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 12/20/2011 18:11'!
test2
	|  t  c surf |
	t := self testText. 
	t addAttribute: (AthensTextBackground new color: (Color green alpha: 0.3 )) from: 1 to: 267.

	c := AthensTextComposer scan: t for: AthensTextScanner new.

	surf := AthensBalloonSurface new form: (Form extent: Display extent depth: 32).

	surf drawDuring: [ :canvas | 
		
		canvas setPaint: (Color white ).
		canvas drawShape: (0@0 corner: 300@300).
		canvas pathTransform scaleBy: 1.

		canvas pathTransform translateX: 300 Y: 0.

		canvas setPaint: Color yellow.
		canvas pathTransform rotateByDegrees: 0.
		c renderOn: canvas
	].

	Display getCanvas translucentImage: surf form at: 0@0

! !

!AthensTextRenderTest classSide methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 10/25/2011 14:52'!
test3
	|  t  c surf |

"t := '12345 pi kl mn op gj the text must flow' "

t := 'qW' 
asText .
	t addAttribute: (TextFontReference toFont: (
LogicalFont
 familyName: 'Arial'
 pointSize: 8
)).

	c := AthensTextComposer scan: t for: AthensTextScanner new.

	surf := AthensBalloonSurface new form: (Form extent: Display extent depth: 32).

	surf drawDuring: [ :canvas | 
		
		canvas setPaint: (Color yellow alpha: 0.5).
		canvas drawShape: (0@0 corner: 300@300).

		canvas pathTransform translateBy: 200@200.
				
"		canvas pathTransform rotateByDegrees: 90. "
		canvas pathTransform scaleBy: 4.
	1 to: 10 do: [:i |
		
		canvas setPaint: (Color blue alpha: 0.5).
		canvas drawShape: (0@0 corner: 100@10).
		
		c renderOn: canvas.
		
		canvas pathTransform rotateByDegrees: 5. 
		canvas pathTransform translateBy: 0 @11 .
	].
	].
	Display getCanvas translucentImage: surf form at: 100@100

! !

!AthensTextRenderTest classSide methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 10/25/2011 19:22'!
test4
	|  t  c surf |

"t := '12345 pi kl mn op gj the text must flow' "

t := 'Unfortunately this class has not been documented yet.' 
asText .
	t addAttribute: (TextFontReference toFont: (
LogicalFont
 familyName: 'Tahoma'
 pointSize: 10
)).
	t addAttribute: (TextColor new color: Color red) from: 3 to: 10.


	c := AthensTextComposer scan: t for: AthensTextScanner new.

	surf := AthensBalloonSurface new form: (Form extent: Display extent depth: 32).

	surf drawDuring: [ :canvas | 
		
		canvas setPaint: (Color yellow alpha: 0.5).
		canvas drawShape: (0@0 corner: 300@300).

		canvas pathTransform scaleBy: 1.
		1 to: 20 do: [:i |
			c renderOn: canvas.
			canvas pathTransform translateBy: 0.1@10.
		]
	].
	Display getCanvas translucentImage: surf form at: 100@100

! !

!AthensTextRenderTest classSide methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 10/25/2011 19:32'!
test5
	|  t  c surf |

"t := '12345 pi kl mn op gj the text must flow' "

t := 'U' 
asText .
	t addAttribute: (TextFontReference toFont: (
LogicalFont
 familyName: 'Tahoma'
 pointSize: 10
)).
	c := AthensTextComposer scan: t for: AthensTextScanner new.

	surf := AthensBalloonSurface new form: (Form extent: Display extent depth: 32).

	surf drawDuring: [ :canvas | 
		
		canvas setPaint: (Color yellow alpha: 0.5).
		canvas drawShape: (0@0 corner: 300@300).

		canvas pathTransform scaleBy: 1.
		1 to: 20 do: [:i |
			c renderOn: canvas.
			canvas pathTransform translateBy: 10@0.1.
		]
	].
	Display getCanvas translucentImage: surf form at: 100@100

! !

!AthensTextRenderTest classSide methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 10/19/2011 14:39'!
testBlitting
	| fnt form blt |

	fnt := LogicalFont
	 familyName: 'Arial'
 	pointSize: 24.

	form := fnt realFont
		glyphOf: $W
				destDepth: 8
				colorValue: 1
				subpixelPosition: 0.


	blt := BitBlt toForm: Display.
	
	blt 
		sourceForm: form;
		destOrigin: 0@0;
		sourceOrigin: 0@0;
		halftoneForm: nil;
		combinationRule: 41;
		width: form width;
		height: form height;
		
		copyBitsColor: 0
		alpha: 255
		gammaTable:  FreeTypeSettings current gammaTable
		ungammaTable: FreeTypeSettings current gammaInverseTable.
		
"	blt copyForm: form to: 100@100 rule: 41."
	Display forceToScreen.
	
"
	Display getCanvas translucentImage: form at: 0@0.		

"
	
	
! !

!AthensTextRenderTest classSide methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 11/10/2011 12:06'!
testText
	|  t  |

t := 'The safety constraint is that the garbage collector 
MUST NOT collect any storage that then becomes needed to continue correct execution of the program. Should weak references 
be added, then garbage collection decisions become observable. 
Based on a suggestion from Cameron McCormack, we state the 
safety constraint as follows: So long as operational semantics of 
the remainder of the program execution includes the possibility that 
a reference to an object X may be dereferenced, then X MUST NOT 
be collected. Thus, the garbage collector is allowed to ignore any 
references that remain present in the semantic state, but which it 
can ascertain will never be dereferenced in any possible execution. 
This sets an upper bound on what state MAY be collected. Put 
another way, if the garbage collector ever reports that X has been 
collected, such as by nullifying a weak reference to X, if operational 
semantics of remaining execution requires the traversal of a strong 
(non-weak) reference to X, then the previous report demonstrates a 
safety violation. 
' 
asText .
	t addAttribute: (TextFontReference toFont: (
LogicalFont
 familyName: 'Arial'
 pointSize: 8
)).
	t addAttribute: (TextEmphasis italic) from: 5 to: 100.
	t addAttribute: (TextColor new color: (Color red)) from: 10 to: 60.
	t addAttribute: (TextEmphasis italic) from: 145 to: 200.
	t addAttribute: (TextColor new color: Color green) from: 230 to: 360.

	t addAttribute: (
		TextFontReference toFont: ( LogicalFont familyName: 'Tahoma'  pointSize: 14)) from: 98 to: 200.

	^ t! !

!AthensTextRenderTest classSide methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 11/21/2011 16:25'!
testWindow

	^ (AthensPluggableTextMorph
		on: self
		text: #testText
		accept: nil) embeddedInMorphicWindowLabeled: 'A fancy looking title text'
! !

!AthensTextRenderTest classSide methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 2/3/2012 17:43'!
testWindowRender
	| surf |
	surf := AthensBalloonSurface new form: (Form extent: Display extent depth: 32).

	surf drawDuring: [ :canvas | 
		
		canvas pathTransform scaleBy: 2.5.
		canvas fullDrawMorph: self testWindow.

	].

	Display getCanvas translucentImage: surf form at: 0@0

! !

Object subclass: #AthensTextRenderer
	instanceVariableNames: 'text font canvas glyphRenderer color advance maxAscent currentLine'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Text'!

!AthensTextRenderer commentStamp: 'IgorStasenko 10/20/2011 10:14' prior: 0!
This is a text renderer, which pipelining a text command(s) , produced by AthensTextComposer
into a concrete calls to canvas.

Its using a glyph renderer instance(s) to draw separate font glyphs on a surface in a most suitable/efficient
manner for given Athens surface.
!

!AthensTextRenderer methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 10/20/2011 13:07'!
initialize
	color := Color black.! !

!AthensTextRenderer methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 3/8/2012 13:33'!
render: aText lines: lines on: aCanvas

	text := aText.
	canvas := aCanvas.
	
	canvas pathTransform restoreAfter: [

		lines do: [:line | | origY |
			currentLine := line.
			advance := 0@0.
			maxAscent := line maxAscent.
			line renderOn: self.
			canvas pathTransform translateX: 0 Y: line heightAndDescent.

		] ]! !

!AthensTextRenderer methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 11/7/2011 14:44'!
renderBackground: bkObj
"self halt."
	canvas setPaint: bkObj color.
	canvas drawShape: ( bkObj start@0 corner: bkObj start + bkObj width @ currentLine heightAndDescent)
! !

!AthensTextRenderer methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 11/7/2011 14:41'!
renderBackground: aColor width: w
"self halt."
	canvas setPaint: aColor.
	canvas drawShape: ( advance x@0 corner: advance x + w +1@ currentLine heightAndDescent)
! !

!AthensTextRenderer methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 10/25/2011 18:31'!
renderCharactersFrom: start to: stop

	glyphRenderer renderGlyphsIn: text from: start to: stop.
	" retrieve a cumulative horizontal advance"
	advance := glyphRenderer getAdvance.! !

!AthensTextRenderer methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 10/20/2011 15:54'!
setCurrentColor: aColor

	color = aColor ifTrue: [ ^ self ].
	color := aColor.
	canvas setPaint: aColor.
	glyphRenderer ifNotNil:  [ glyphRenderer setColor: aColor ].! !

!AthensTextRenderer methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 10/28/2011 11:19'!
setCurrentFont: aFont
	font := aFont.
	glyphRenderer := font glyphRendererOn: canvas surface.
	glyphRenderer 
		setPosition: 0@0 advance: advance baseline: maxAscent;
		setColor: color.! !

Object subclass: #AthensTextScanner
	instanceVariableNames: 'paragraph text currentLine lines currentAttributes currentFont currentColor textStyle currentStrip currentBackground'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Text'!

!AthensTextScanner commentStamp: 'IgorStasenko 2/6/2012 16:22' prior: 0!
I responsible for scanning the Text and transforming it into a list of AthensTextLine, where each line is a list of AthensTextDisplayCommand (sub)instances!

!AthensTextScanner methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 11/10/2011 12:00'!
addCharacter: position
	| charWidth fnt |

	currentStrip ifNil: [

		self applyAttributes.
		
		fnt := currentFont font.
		currentLine maxAscent: fnt getPreciseAscent.
		currentLine maxDescent: fnt getPreciseDescent.
		currentLine maxHeight: fnt getPreciseHeight.
		
		currentStrip := AthensCharacterSpan new.
		currentLine addCommand: currentStrip.
		currentStrip start: position.
	].
	
	charWidth := currentFont font getGlyphWidth: (text at: position).
	currentLine addWidth: charWidth.
	currentBackground ifNotNil: [
		currentBackground addWidth: charWidth ].
	
	currentStrip stop: position.
! !

!AthensTextScanner methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 11/10/2011 11:26'!
addCommand: aCommand
	currentLine addCommand: aCommand! !

!AthensTextScanner methodsFor: 'text attributes' stamp: 'IgorStasenko 11/10/2011 11:14'!
addEmphasis: emphasis
	| fnt |
	
	fnt := currentFont ifNil: [ self defaultFont ] ifNotNil: [ currentFont font ].
	
	self setActualFont: (fnt emphasized: emphasis)! !

!AthensTextScanner methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 11/10/2011 11:58'!
applyAttributes
	currentAttributes do: [:attr |
		attr emphasizeScanner: self ]. 
	
	currentFont ifNil: [ self setActualFont: self defaultFont ].
	currentColor ifNil: [ self textColor: self defaultColor ].
	! !

!AthensTextScanner methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 11/10/2011 11:13'!
defaultColor
	^ Color black! !

!AthensTextScanner methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 11/10/2011 11:13'!
defaultFont

	^ self textStyle defaultFont! !

!AthensTextScanner methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 10/11/2011 16:53'!
defaultTextStyle

	^ TextStyle default! !

!AthensTextScanner methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 10/11/2011 17:19'!
endLine: index

	currentLine  endIndex: index! !

!AthensTextScanner methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 11/10/2011 11:54'!
initialize
	lines := OrderedCollection new.
	currentAttributes := #().! !

!AthensTextScanner methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 10/17/2011 12:45'!
lines
	^ lines! !

!AthensTextScanner methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 11/10/2011 12:21'!
newLine: startingIndex 

	currentLine := lines add: (AthensTextLine new text: text;
		startIndex: startingIndex; yourself).

"	currentStrip ifNotNil: [
		
		currentLine addCommand: currentFont copy.
		currentLine addCommand: currentColor copy.

		currentBackground ifNotNil: [
			currentBackground := currentBackground copy start: 0; width: 0. 
			currentLine addCommand: currentBackground ].
	].
"
"reset everything, to make sure all attributes will be reapplied for new line"
	
	currentColor := currentBackground := currentFont := currentStrip := nil.! !

!AthensTextScanner methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 10/28/2011 14:58'!
resetBackground
	self textBackground: Color transparent.! !

!AthensTextScanner methodsFor: 'text attributes' stamp: 'IgorStasenko 11/10/2011 12:04'!
setActualFont: aFont

	currentFont ifNil: [
		
		currentFont := AthensFontChange new font: aFont.
		self addCommand: currentFont.
		^ self 
	].

	aFont = currentFont font ifFalse: [
		currentFont := AthensFontChange new font: aFont.
		self addCommand: currentFont.
	].
! !

!AthensTextScanner methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 11/10/2011 12:13'!
setAttributes: anArray 
	currentBackground := nil.
	currentAttributes := anArray.
	
	"we should reset character string here"
	currentStrip := nil.
! !

!AthensTextScanner methodsFor: 'text attributes' stamp: 'IgorStasenko 10/11/2011 16:52'!
setFont: fontNumber

	self setActualFont: (self textStyle fontAt: fontNumber).
	
	! !

!AthensTextScanner methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 10/11/2011 16:08'!
text: aText 
	text := aText! !

!AthensTextScanner methodsFor: 'text attributes' stamp: 'IgorStasenko 11/10/2011 11:31'!
textBackground: aColor

	"background is transparent by default "
	
	(currentBackground notNil and: [ currentBackground color = aColor ])
		ifTrue: [ ^ self].
		
	currentBackground ifNil: [ aColor isTransparent ifTrue: [ ^ self ] ].
	
	currentBackground := AthensBackgroundChange new color: aColor.
	currentBackground start: currentLine width.
	currentLine addCommand: currentBackground.! !

!AthensTextScanner methodsFor: 'text attributes' stamp: 'IgorStasenko 11/10/2011 11:15'!
textColor: aColor

	currentColor ifNil: [
		currentColor := AthensColorChange new color: aColor.
		self addCommand: currentColor.
		^ self 
	].

	currentColor color = aColor ifFalse: [
		currentColor := AthensColorChange new color: aColor.
		self addCommand: currentColor.	
	].
	! !

!AthensTextScanner methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 10/11/2011 16:53'!
textStyle
	^ textStyle ifNil: [ textStyle := self defaultTextStyle ]! !

Object subclass: #GlyphRenderer
	instanceVariableNames: 'font'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Athens-Text'!

!GlyphRenderer classSide methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 7/12/2011 19:45'!
forFont: aFont
	^ self basicNew initForFont: aFont! !

!GlyphRenderer classSide methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 7/12/2011 19:26'!
test

	| fnt glyph |
	
	fnt := StandardFonts defaultFont.
	fnt class == LogicalFont ifTrue: [
		fnt := fnt realFont.
		].
	fnt class ~~ FreeTypeFont ifTrue: [ self error: 'invalid font' ].
	
	glyph := self  new initPixelSize: fnt pixelSize.
	
	glyph renderGlyph: $A asUnicode face: fnt face.
	glyph renderGlyph: $B asUnicode face: fnt face.
	glyph renderGlyph: $C asUnicode face: fnt face.
	glyph renderGlyph: $+ asUnicode face: fnt face.
	
	glyph drawOn: Display at: 0@0! !

!GlyphRenderer methodsFor: 'as yet unclassified' stamp: 'IgorStasenko 7/12/2011 19:45'!
initForFont: aFont

	self subclassResponsibility ! !
